package org.jboss.qa.staxsource;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;

@JsonIgnoreProperties(ignoreUnknown = true)
@XmlRootElement(namespace="urn:jboss-soa:order-service:1.0")
public class SimpleOrder implements Serializable {


    private static final long serialVersionUID = -1815063706242924535L;

    private Long id;
    private String name;
    private List<OrderItem> items;

    public SimpleOrder() {}


    // Simple constructor for sample message
    public SimpleOrder(Long id, Long firstSku) {
        super();
        this.id = id;
        this.name = "Sample Order " + id;
        List<OrderItem> tmpItems = new ArrayList<OrderItem>();
        tmpItems.add(new OrderItem(firstSku, 1, 101.10));
        tmpItems.add(new OrderItem(11l, 2, 202.20));
        tmpItems.add(new OrderItem(12l, 3, 303.30));
        tmpItems.add(new OrderItem(13l, 4, 404.40));
        setItems(tmpItems);
    }

    public SimpleOrder(Long id, String name, List<OrderItem> items) {
        super();
        this.id = id;
        this.name = name;
        setItems(items);
    }

    public SimpleOrder(Long id) {
        this(id, 10l);

    }

    @XmlElement(name = "item", namespace="urn:jboss-soa:order-service:1.0")
    @XmlElementWrapper(name = "items", namespace="urn:jboss-soa:order-service:1.0")
    public List<OrderItem> getItems() {
        return items;
    }

    public void setItems(List<OrderItem> items) {
        this.items = items;
    }

    @XmlElement(namespace="urn:jboss-soa:order-service:1.0")
    public Integer getQuantity() {
        return items.size();
    }

    @XmlAttribute
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    @XmlElement(namespace="urn:jboss-soa:order-service:1.0")
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @JsonIgnore
    public BigDecimal getTotalPrice() {
        BigDecimal totalPrice = new BigDecimal(0);
        if(items != null){
            for (OrderItem item : items) {
                totalPrice = totalPrice.add(item.getTotalPrice());
            }
        }
        return totalPrice;
    }


    @Override
    public String toString() {
        return "SimpleOrder [id=" + id + ", name=" + name + ", items=" + items
                + ", totalPrice=" + getTotalPrice() + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        return result = prime * result + toString().hashCode();
    }


    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        SimpleOrder other = (SimpleOrder) obj;
        return this.toString().equals(other.toString());
    }
}
